<?php
namespace Zeedhi\DevOps\Model\Devops\Repositories;

use Doctrine\DBAL\Connection;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\ProductFiles;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Mnemonic extends \Doctrine\ORM\EntityRepository {

    public function findMnemonicByProductFiles($productFileId, $filter = " "){
        $query = <<<SQL
         SELECT * 
           FROM (SELECT MNE.ID, MNE.MNEMONIC, MNE.LABEL, MNE.ORGANIZATION_ID, MNE.CREATED, MNE.CREATED_BY,
                        MNE.UPDATED, MNE.UPDATED_BY, MNE.IS_ACTIVE, MNE.IS_ENCRYPTED, 
                        CASE WHEN MNE.IS_ENCRYPTED = 'Y' AND UPPER(MNE.DEFAULT_VALUE) <> 'NULL'
                             THEN '**********' 
                             ELSE MNE.DEFAULT_VALUE 
                         END AS DEFAULT_VALUE, MNE.MNEMONIC_GROUPING_ID, MG.NAME AS GROUP_NAME
                   FROM PRODUCT_FILES_MNEMONIC PFM
                   JOIN MNEMONIC MNE      ON (MNE.ID = PFM.MNEMONIC_ID)
                   JOIN PRODUCT_FILES PFL ON PFL.ID = PFM.PRODUCT_FILES_ID
                   LEFT JOIN MNEMONIC_GROUPING MG ON MG.ID = MNE.MNEMONIC_GROUPING_ID
                  WHERE PFM.PRODUCT_FILES_ID = :PRODUCT_FILE_ID
                    AND PFL.TYPE NOT IN (:EXCLUDED_TYPES))
          WHERE 1 = 1
                $filter
                ORDER BY MNEMONIC ASC
SQL;
        $params = array(
            'PRODUCT_FILE_ID' => $productFileId,
            'EXCLUDED_TYPES' => array(
                ProductFiles::CONFIGURATION,
                ProductFiles::OTHERS_FILES,
            )
        );
        $paramTypes = array(
            'EXCLUDED_TYPES' => Connection::PARAM_STR_ARRAY,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }
    public function findMnemonicByProductFilesAndServer($productFileId, $serverId){
        $query = <<<SQL
			 SELECT MNE.MNEMONIC, NVL(SMN.VALUE,MNE.DEFAULT_VALUE) AS VALUE, MNE.IS_ENCRYPTED
               FROM PRODUCT_FILES_MNEMONIC PFM
          LEFT JOIN MNEMONIC MNE ON(MNE.ID = PFM.MNEMONIC_ID)
          LEFT JOIN SERVER_MNEMONIC SMN ON(SMN.MNEMONIC_ID = MNE.ID AND SMN.SERVER_ID = :SERVER_ID)
              WHERE PFM.PRODUCT_FILES_ID = :PRODUCT_FILE_ID
SQL;
        $params = array(
            'PRODUCT_FILE_ID'=> $productFileId,
            'SERVER_ID'=> $serverId,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findMnemonicNotValueByProductFilesAndServer($productFileIds,$serverId){
        $query = <<<SQL
			 SELECT MNE.MNEMONIC, NVL(SMN.VALUE, MNE.DEFAULT_VALUE) AS VALUE
               FROM PRODUCT_FILES_MNEMONIC PFM
          LEFT JOIN MNEMONIC MNE ON(MNE.ID = PFM.MNEMONIC_ID)
          LEFT JOIN SERVER_MNEMONIC SMN ON (SMN.MNEMONIC_ID = MNE.ID AND SMN.SERVER_ID = :SERVER_ID)
              WHERE PFM.PRODUCT_FILES_ID IN (:PRODUCT_FILE_ID)
                AND SMN.VALUE IS NULL AND MNE.DEFAULT_VALUE IS NULL
SQL;
        $params = array(
            'SERVER_ID'=> $serverId,
            'PRODUCT_FILE_ID'=> $productFileIds,
        );
        $paramTypes = array(
            'PRODUCT_FILE_ID' => Connection::PARAM_INT_ARRAY,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
    }
    
     public function validateProductServerLinked($serverId, $productId) {
        $query = <<<SQL
			    SELECT PS.PRODUCT_ID
                  FROM PRODUCT_SERVER PS
                 INNER JOIN SERVER SR ON SR.ID = PS.SERVER_ID
                 WHERE PS.SERVER_ID IN (:SERVER_ID)
                   AND PS.PRODUCT_ID = :PRODUCT_ID
                   AND PS.IS_ACTIVE  = :IS_ACTIVE
                   AND SR.IS_ACTIVE  = :IS_ACTIVE
              GROUP BY PS.PRODUCT_ID
SQL;
        $params = array(
            'SERVER_ID'  => $serverId,
            'PRODUCT_ID' => $productId,
            'IS_ACTIVE'  => 'Y'
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
   

    public function validateMnemonicWithLabelByProductFileId($productFileId){
        $query = <<<SQL
       SELECT MNE.ID, MNE.LABEL
         FROM MNEMONIC MNE
         JOIN PRODUCT_FILES_MNEMONIC PFM ON(PFM.MNEMONIC_ID = MNE.ID)
        WHERE PFM.PRODUCT_FILES_ID = :productFilesId
          AND MNE.LABEL IS NULL
SQL;

        $params = array(
            'productFilesId'=> $productFileId,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findMnemonicByServer($serverId, $conditions = "", $filterWhereInByProductsId = "", $limit = null, $offset = null){
        $query = <<<SQL
        SELECT MG.NAME AS MNEMONIC_GROUPING_NAME, MNM.* 
          FROM (SELECT M.SERVER_ID, M.MNEMONIC, M.LABEL, M.MNEMONIC_ID, M.IS_DEFAULT_VALUE, M.VALUE, M.IS_ENCRYPTED, M.MNEMONIC_GROUPING_ID, M.MNEMONIC_GROUP_NAME,
              (SELECT RTRIM(XMLAGG(XMLELEMENT(E,NAME,',').EXTRACT('//text()') ORDER BY NAME).GETCLOBVAL(),',') AS PRODUCT_NAME
                FROM (SELECT PROD.NAME
                        FROM PRODUCT_FILES_MNEMONIC PFM
                        JOIN MNEMONIC MN ON PFM.MNEMONIC_ID = MN.ID
                        JOIN PRODUCT_FILES PF ON PF.ID = PFM.PRODUCT_FILES_ID
                        JOIN PRODUCT PROD ON PROD.ID = PF.PRODUCT_ID
                       WHERE MN.ID = M.MNEMONIC_ID
                    GROUP BY PROD.NAME)
              ) AS PRODUCT_NAME
         FROM (
               SELECT  NVL(MN.IS_ENCRYPTED,SMN.IS_ENCRYPTED) AS IS_ENCRYPTED, NVL(MN.SERVER_ID,SMN.SERVER_ID) AS SERVER_ID, NVL(MN.MNEMONIC,SMN.MNEMONIC) AS MNEMONIC,
                       NVL(MN.LABEL,SMN.LABEL) AS LABEL, MN.PRODUCT_ID, MN.MNEMONIC_GROUPING_ID, MN.MNEMONIC_GROUP_NAME, MN.MNEMONIC_GROUP_NAME AS MNEMONIC_GROUPING_NAME,
                       NVL(MN.MNEMONIC_ID,SMN.MNEMONIC_ID) AS MNEMONIC_ID, MN.PRODUCT_NAME,
                       (CASE WHEN NVL(MN.IS_ENCRYPTED,SMN.IS_ENCRYPTED) = 'Y' AND UPPER(NVL(MN.VALUE,NVL(SMN.VALUE, MN.DEFAULT_VALUE))) <> 'NULL'
                             THEN '**********'
                             ELSE (CASE WHEN NVL(MN.VALUE,SMN.VALUE) IS NOT NULL
                                        THEN NVL(MN.VALUE,SMN.VALUE)
                                        WHEN (NVL(MN.VALUE,SMN.VALUE) IS NULL) AND (MN.DEFAULT_VALUE IS NOT NULL)
                                        THEN MN.DEFAULT_VALUE
                                        ELSE NVL(MN.VALUE,SMN.VALUE)
                                    END)
                              END) AS VALUE,
                       (CASE WHEN (NVL(MN.VALUE,SMN.VALUE) IS NOT NULL) AND (MN.DEFAULT_VALUE IS NOT NULL)
                             THEN 'M'
                             WHEN NVL(MN.VALUE,SMN.VALUE) IS NOT NULL
                             THEN 'N'
                             WHEN (NVL(MN.VALUE,SMN.VALUE) IS NULL) AND (MN.DEFAULT_VALUE IS NOT NULL)
                             THEN 'Y'
                             ELSE 'N'
                         END) AS IS_DEFAULT_VALUE
                 FROM (
                      SELECT PMN.IS_ENCRYPTED, PMN.MNEMONIC_GROUPING_ID, SER.ID AS SERVER_ID, SER.NAME, PMN.MNEMONIC, PMN.MNEMONIC_LABEL AS LABEL,
                             PRO.NAME AS PRODUCT_NAME, SEM.VALUE, PMN.MNEMONIC_ID, PMN.DEFAULT_VALUE, PRO.ID AS PRODUCT_ID, PMN.MNEMONIC_GROUP_NAME
                        FROM PRODUCT_SERVER PSE
                        JOIN SERVER SER ON (SER.ID = PSE.SERVER_ID)
                        JOIN (SELECT MG.NAME AS MNEMONIC_GROUP_NAME, PFI.STATE, MNE.IS_ENCRYPTED, MNE.MNEMONIC_GROUPING_ID, PFI.PRODUCT_ID, MNE.DEFAULT_VALUE, MNE.MNEMONIC, MNE.LABEL AS MNEMONIC_LABEL, MNE.ID AS MNEMONIC_ID, PFI.TYPE,
                                    (CASE WHEN PFI.TYPE IN('A','B') THEN 'T'
                                          WHEN PFI.TYPE IN('L','K','J','I') THEN 'P'
                                          WHEN PFI.TYPE IN('D','P','H','S') THEN 'C'
                                          ELSE NULL
                                      END) AS TYPE_SERVER_USER
                                FROM PRODUCT_FILES_MNEMONIC PFM
                                JOIN MNEMONIC MNE ON (MNE.ID = PFM.MNEMONIC_ID)
                                JOIN PRODUCT_FILES PFI ON (PFI.ID = PFM.PRODUCT_FILES_ID)
                                LEFT JOIN MNEMONIC_GROUPING MG ON (MNE.MNEMONIC_GROUPING_ID = MG.ID)) PMN
                          ON (PMN.PRODUCT_ID = PSE.PRODUCT_ID AND PMN.TYPE_SERVER_USER = PSE.USETYPE)
                        JOIN PRODUCT PRO ON (PRO.ID = PMN.PRODUCT_ID AND PRO.IS_ACTIVE = :IS_ACTIVE)
                   LEFT JOIN SERVER_MNEMONIC SEM ON (SEM.SERVER_ID = SER.ID AND SEM.MNEMONIC_ID = PMN.MNEMONIC_ID)
                   LEFT JOIN MNEMONIC MNE ON (MNE.ID = SEM.MNEMONIC_ID)
                       WHERE SER.ID = :SERVER_ID
                       AND (PMN.STATE = 'A' OR SER.ONLY_APPROVED_SCRIPTS = 'N')
                    GROUP BY PMN.MNEMONIC_GROUPING_ID, PMN.MNEMONIC_GROUP_NAME, PMN.IS_ENCRYPTED, SER.ID, SER.NAME, PMN.MNEMONIC, PMN.MNEMONIC_LABEL, PRO.NAME, PRO.ID,
                             SEM.VALUE, PMN.MNEMONIC_ID, PMN.DEFAULT_VALUE
                    ORDER BY PMN.MNEMONIC_ID)MN
                   FULL JOIN (SELECT MNE.IS_ENCRYPTED, SEM.SERVER_ID, MNE.MNEMONIC, MNE.LABEL, SEM.VALUE, SEM.MNEMONIC_ID
                                FROM SERVER_MNEMONIC SEM
                           LEFT JOIN MNEMONIC MNE ON (SEM.MNEMONIC_ID = MNE.ID)
                               WHERE SEM.SERVER_ID = :SERVER_ID) SMN ON (SMN.MNEMONIC_ID = MN.MNEMONIC_ID)
                   WHERE 1=1
                           $filterWhereInByProductsId
                GROUP BY MN.IS_ENCRYPTED, MN.MNEMONIC_GROUPING_ID, MN.MNEMONIC_GROUP_NAME,SMN.IS_ENCRYPTED, MN.SERVER_ID, MN.NAME, MN.MNEMONIC, MN.LABEL, MN.VALUE, MN.MNEMONIC_ID,
                         SMN.MNEMONIC_ID, SMN.VALUE, SMN.LABEL, SMN.MNEMONIC, SMN.SERVER_ID,
                         MN.PRODUCT_NAME, MN.DEFAULT_VALUE, MN.PRODUCT_ID)M
           WHERE 1=1
                 $conditions
        GROUP BY M.MNEMONIC_GROUPING_ID, M.MNEMONIC_GROUP_NAME, M.IS_ENCRYPTED, M.SERVER_ID, M.MNEMONIC, M.LABEL, M.VALUE, M.MNEMONIC_ID, M.IS_DEFAULT_VALUE
        ORDER BY M.MNEMONIC, M.MNEMONIC_ID DESC) MNM
        JOIN MNEMONIC MNE ON MNE.ID = MNM.MNEMONIC_ID
        LEFT JOIN MNEMONIC_GROUPING MG ON MNE.MNEMONIC_GROUPING_ID = MG.ID
        
        ORDER BY MNM.MNEMONIC ASC

SQL;
        $params = array(
            'SERVER_ID'=> $serverId,
            'IS_ACTIVE'=> Utilities::IS_ACTIVE_YES,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
    
}